SystemOrganization addCategory: #ModularAgency!


Object subclass: #AgencyComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModularAgency'!

AgencyComponent class
	instanceVariableNames: ''!

AgencyComponent subclass: #AgencyLogicSystem
	instanceVariableNames: 'agency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModularAgency'!

AgencyLogicSystem subclass: #ModularAgencyKnowHow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModularAgency'!

AgencyLogicSystem class
	instanceVariableNames: ''!

ModularAgencyKnowHow class
	instanceVariableNames: ''!

AgencyLogicSystem subclass: #ModularAgencyGetter
	instanceVariableNames: 'permaCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModularAgency'!

ModularAgencyGetter class
	instanceVariableNames: ''!

AgencyComponent subclass: #ModularAgency
	instanceVariableNames: 'locus dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ModularAgency'!

ModularAgency class
	instanceVariableNames: ''!


!AgencyComponent commentStamp: 'spfa 10/13/2019 14:58' prior: 0!
AgencyComponent subclasses are components of a ModularAgency (an 'agency').

Each component takes a specific #responsibility. 

Responsibilities should be orthogonal in scope, i.e. two different responsibilities should not cover the same conceptual ground. There is at most one component for any specific responsibility in an agency, although monadic logic systems can have their own shadowing components.
!
]style[(406)!

!AgencyLogicSystem commentStamp: 'spfa 10/13/2019 14:54' prior: 0!
An AgencyLogicSystem is a an agency component with access to the agency. 

It is suitable for encoding the internal logic of the agency, i.e. how components interface with each other, while non-logic AgencyComponent subclasses are suitable for  autonomous data structures.

A logic system can become a monad, which means it can have access to its own version of some or all of the other components in the agency. Precisely, the logic system agency becomes a child of the agency that contains the logic system.
!
]style[(510)!

!ModularAgency commentStamp: 'spfa 10/15/2019 11:26' prior: 0!
A ModularAgency (an 'agency') maintains a collection of AgencyComponents, each of them having a specific responsibility.

Notable components are logic systems: an AgencyLogicSystem is a component with a reference to an agency. That agency is either the agency that contains it, or a child of that agency (a 'lower' agency), in which case it is called a 'monad'.

A ModularAgency can itself be a component in an other agency. It is not a logic system, so if it wants a reference to the agency it is a component of, then it must includes this agency as one of its own components.!
]style[(577)!

!ModularAgencyGetter commentStamp: 'spfa 10/16/2019 17:01' prior: 0!
A ModularAgencyGetter is an agency logic system handling redirections.!
]style[(70)!

!ModularAgencyKnowHow commentStamp: 'spfa 10/16/2019 17:04' prior: 0!
A SpriteKnowHow handles high-level requests and transform them into a configuration of components. 

It can be considered as a pluggable library of scripts.!
]style[(156)!

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/15/2019 12:07'!
addComponent: anAgencyComponent

	^ self addComponent: anAgencyComponent 
		responsibleFor: anAgencyComponent nominalResponsibility
! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/16/2019 09:33'!
addComponent: anAgencyComponent responsibleFor: aSymbol

	self removeComponentResponsibleFor: aSymbol.
	^ dictionary 
		at: aSymbol
		put: (anAgencyComponent agency: self).
! !

!AgencyLogicSystem methodsFor: 'redirection operators' stamp: 'spfa 10/16/2019 21:33'!
agdo

	^ self agency do! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/13/2019 11:58'!
agency

	^ agency! !

!AgencyComponent methodsFor: 'accessing' stamp: 'spfa 10/15/2019 11:22'!
agency: ignored

	"A component that is not a logic system does not need to have a reference to a specific agency"! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/13/2019 12:42'!
agency: aSpriteGameAgent

	agency ifNotNil: [self detachFromAgency: agency].

	agency := aSpriteGameAgent! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/13/2019 12:20'!
agent

	"The agent is the top-level agency. Its locus is not an agency."

	^ self agency agent! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/13/2019 10:23'!
agent

	"The agent is the top-level agency"

	^ locus isAgency ifTrue: [locus agent] ifFalse: [self]! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/15/2019 10:54'!
agentLocus

	^ self agent locus	! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/15/2019 10:54'!
agentLocus: anObject

	self higherAgency ifNotNil: [^ self higherAgency sprite: anObject].

	locus := anObject! !

!AgencyLogicSystem methodsFor: 'redirection operators' stamp: 'spfa 10/16/2019 21:33'!
agt

	^ self agency get! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 11/21/2019 10:55'!
autoInstallingComponentFor: aSymbol

	"Return the component with responsibility aSymbol
	If none, install one as specified in the constitution
	If none specified, do not do anything - there will be no such component"

	^ self componentFor: aSymbol 
		ifNonePut: [(self constitution dictionary at: aSymbol ifAbsent: [^ nil]) value]! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/16/2019 12:01'!
beMonad

	(self agency ifNil: [self error: 'Agency needed for monadic involution'])
		installDerivativeAgency! !

!ModularAgencyKnowHow methodsFor: 'meta' stamp: 'spfa 10/16/2019 17:05'!
browse

	self browse! !

!ModularAgencyGetter methodsFor: 'accessing' stamp: 'spfa 10/17/2019 18:48'!
clearPermaCache

	permaCache removeAll! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/11/2019 11:40'!
componentFor: aSymbol

	^ self componentFor: aSymbol ifNone: [nil]! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:32'!
componentFor: aSymbol ifNone: aBlock

	^ dictionary at: aSymbol ifAbsent: [
		| np1 |
		(np1 := self higherAgency)
			ifNotNil: [np1 componentFor: aSymbol ifNone: aBlock]
			ifNil: aBlock]! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:32'!
componentFor: aSymbol ifNonePut: aBlock

	"If the component is missing, the default one will be installed in the top agency
	see also #componentFor:ifNonePutLocally: "

	^ dictionary at: aSymbol ifAbsent: [
		| np1 |
		(np1 := self higherAgency)
			ifNotNil: [np1 componentFor: aSymbol ifNonePut: aBlock]
			ifNil: [self agent installComponent: aBlock value]]
! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 13:29'!
componentFor: aSymbol ifNonePutLocally: aBlock

	self higherAgency ifNotNil: [
		^ self componentFor: aSymbol ifNone: [	self installComponent: aBlock value]].

	^ self componentFor: aSymbol ifNonePut: aBlock! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:32'!
components

	^ dictionary values! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:36'!
componentsIncludes: anAgencyComponent

	^ dictionary includesIdentity: anAgencyComponent! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/16/2019 09:58'!
componentsIncludesResponsibility: aSymbol

	^ dictionary includesKey: aSymbol! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 11/21/2019 10:33'!
constitutionAt: aSymbol

	^ self constitution constitutionAt: aSymbol! !

!ModularAgency methodsFor: 'acessing components' stamp: 'spfa 10/15/2019 13:55'!
defaultGetter

	^ self subclassResponsibility! !

!ModularAgency methodsFor: 'acessing components' stamp: 'spfa 10/15/2019 13:55'!
defaultKnowHow

	^ self subclassResponsibility! !

!AgencyComponent methodsFor: 'printing' stamp: 'spfa 10/10/2019 14:48'!
describeYourselfOn: aStream! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/13/2019 11:44'!
detachFromAgency: aModulatAgency
! !

!ModularAgency methodsFor: 'acessing components' stamp: 'spfa 10/15/2019 13:53'!
do

	^ self componentFor: #knowhow ifNonePut: [self defaultKnowHow]! !

!ModularAgency methodsFor: 'acessing components' stamp: 'spfa 10/15/2019 13:53'!
get

	^ self componentFor: #get ifNonePut: [self defaultGetter]! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/13/2019 10:19'!
higherAgency

	^ locus isAgency ifTrue: [locus] ifFalse: [nil]! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/13/2019 10:18'!
higherAgency: anAgent

	locus := anAgent! !

!ModularAgency methodsFor: 'initialize-release' stamp: 'spfa 10/13/2019 14:32'!
initialize

	dictionary := IdentityDictionary new! !

!ModularAgencyGetter methodsFor: 'accessing' stamp: 'spfa 10/17/2019 18:48'!
initialize

	super initialize.
	permaCache := IdentityDictionary new! !

!AgencyComponent methodsFor: 'installing' stamp: 'spfa 10/14/2019 09:34'!
initializeAsComponentReplacing: aComponent

	aComponent ifNotNil: #wasReplaced.! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/15/2019 12:06'!
installComponent: anAgencyComponent

	^ self installComponent: anAgencyComponent 
		responsibleFor: anAgencyComponent nominalResponsibility! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/15/2019 14:43'!
installComponent: anAgencyComponent responsibleFor: aSymbol

	| previous |

	previous := dictionary at: aSymbol ifAbsent: [nil].
	self addComponent: anAgencyComponent responsibleFor: aSymbol.
	anAgencyComponent initializeAsComponentReplacing: previous.
	
	^ anAgencyComponent! !

!ModularAgency methodsFor: 'monad support' stamp: 'spfa 10/16/2019 16:41'!
installDerivativeAgency

	"Create an empty child agency and install it as the agency of all installed components"

	| dag |

	dag := self species new higherAgency: self.
	self agent components do: [:ea | ea agency: dag].

	self assert: self agent motor agency == dag.! !

!AgencyComponent methodsFor: 'installing' stamp: 'spfa 10/10/2019 00:31'!
isAboutToBeRemoved! !

!AgencyComponent methodsFor: 'testing' stamp: 'spfa 10/15/2019 11:22'!
isAgency

	^ false! !

!ModularAgency methodsFor: 'testing' stamp: 'spfa 10/13/2019 10:19'!
isAgency

	^ true! !

!Object methodsFor: '*ModularAgency' stamp: 'spfa 10/13/2019 10:20'!
isAgency

	^ false! !

!AgencyComponent methodsFor: 'testing' stamp: 'spfa 10/11/2019 10:31'!
isLogicSystem

	^ false! !

!AgencyLogicSystem methodsFor: 'testing' stamp: 'spfa 10/11/2019 10:31'!
isLogicSystem

	^ true! !

!AgencyLogicSystem methodsFor: 'testing' stamp: 'spfa 10/16/2019 23:53'!
isMonad

	"A logic system can be recognized as a monad when its agency does not feature the system as one of its own components (see #beMonad)"

	^ (self agency componentsIncludes: self) not 
		and: [self agency higherAgency notNil
				"else any sleeping or free component would be considered a monad"]! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:32'!
localComponentFor: aSymbol ifNone: aBlock

	^ dictionary at: aSymbol ifAbsent: aBlock! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:33'!
localResponsibilities

	^ dictionary keys! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/13/2019 14:39'!
localResponsibilitiesIncludes: aSymbol

	^ dictionary identityIncludesKey: aSymbol! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/13/2019 10:57'!
locus

	^ locus! !

!ModularAgency methodsFor: 'accessing' stamp: 'spfa 10/13/2019 10:57'!
locus: anObject

	locus := anObject! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/16/2019 22:59'!
logicSystems

	^ dictionary values select: #isLogicSystem! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/16/2019 22:58'!
logicSystemsDo:  aBlock

	^ dictionary valuesDo: [:ea | ea isLogicSystem ifTrue: [aBlock value: ea]]! !

!AgencyComponent methodsFor: 'accessing' stamp: 'spfa 10/15/2019 11:59'!
nominalResponsibility

	self subclassResponsibility! !

!ModularAgencyGetter methodsFor: 'accessing' stamp: 'spfa 10/15/2019 13:50'!
nominalResponsibility

	^ #getter! !

!ModularAgencyKnowHow methodsFor: 'accessing' stamp: 'spfa 10/15/2019 12:01'!
nominalResponsibility

	^ #knowhow! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/15/2019 12:03'!
removeComponent: anAgencyComponent

	self removeComponentResponsibleFor: anAgencyComponent nominalResponsibility.
! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 11/15/2019 16:45'!
removeComponentResponsibleFor: aSymbol

	(dictionary at: aSymbol ifAbsent: [^ self]) isAboutToBeRemoved.
	dictionary removeKey: aSymbol! !

!AgencyLogicSystem methodsFor: 'agency' stamp: 'spfa 10/20/2019 21:01'!
rootAgency

	"The ground agency is the agency actually featuring the receiver as one of its components - it is only different from the agency per se for monads"

	| ga |

	ga := self agency.
	[ga componentsIncludes: self] whileFalse: [ga := ga higherAgency ifNil: [^ nil]].

	^ ga! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 11/15/2019 16:46'!
uninstallComponent: anAgencyComponent

	self removeComponent: anAgencyComponent.
	anAgencyComponent wasRemoved! !

!ModularAgency methodsFor: 'components' stamp: 'spfa 10/15/2019 12:05'!
uninstallComponentResponsibleFor: aSymbol

	| component |

	component := dictionary at: aSymbol ifAbsent: [nil].
	component isAboutToBeRemoved.
	dictionary removeKey: aSymbol.
	component wasRemoved! !

!AgencyComponent methodsFor: 'installing' stamp: 'spfa 10/16/2019 14:23'!
wasRemoved

	"#wasRemoved is send when the receiver is explicitely removed from its agency..
	See also #wasReplaced"! !

!AgencyComponent methodsFor: 'installing' stamp: 'spfa 10/16/2019 14:34'!
wasReplaced

	"#wasReplaced is send just after the receiver has been automatically removed from its agency, during the installation of a new component.
	See also #wasRemoved"

	self wasRemoved "the default implementation considers that it amounts to the same thing"! !
